Stan Math Library: stan/math/prim/scal/prob/skew_normal_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

skew_normal_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_SKEW_NORMAL_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_SKEW_NORMAL_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
5 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
6 #include <stan/math/prim/scal/err/check_finite.hpp>
7 #include <stan/math/prim/scal/err/check_not_nan.hpp>
8 #include <stan/math/prim/scal/err/check_positive.hpp>
9 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
10 #include <stan/math/prim/scal/fun/owens_t.hpp>
11 #include <stan/math/prim/scal/fun/value_of.hpp>
12 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
13 #include <stan/math/prim/scal/meta/include_summand.hpp>
14 #include <stan/math/prim/scal/fun/constants.hpp>
15 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <boost/math/distributions.hpp>
19 #include <cmath>
20
21 namespace stan {
22
23 namespace math {
24
25 template <typename T_y, typename T_loc, typename T_scale, typename T_shape>
26 typename return_type<T_y, T_loc, T_scale, T_shape>::type
27 skew_normal_cdf_log(const T_y& y, const T_loc& mu, const T_scale& sigma,
28 const T_shape& alpha) {
29 static const char* function("stan::math::skew_normal_cdf_log");
30 typedef typename stan::partials_return_type<T_y, T_loc, T_scale,
31 T_shape>::type
32 T_partials_return;
33
34 using stan::math::check_positive;
35 using stan::math::check_finite;
36 using stan::math::check_not_nan;
37 using stan::math::value_of;
38 using stan::math::check_consistent_sizes;
39 using stan::math::owens_t;
40
41 T_partials_return cdf_log(0.0);
42
43 // check if any vectors are zero length
44 if (!(stan::length(y)
45 && stan::length(mu)
46 && stan::length(sigma)
47 && stan::length(alpha)))
48 return cdf_log;
49
50 check_not_nan(function, "Random variable", y);
51 check_finite(function, "Location parameter", mu);
52 check_not_nan(function, "Scale parameter", sigma);
53 check_positive(function, "Scale parameter", sigma);
54 check_finite(function, "Shape parameter", alpha);
55 check_not_nan(function, "Shape parameter", alpha);
56 check_consistent_sizes(function,
57 "Random variable", y,
58 "Location parameter", mu,
59 "Scale parameter", sigma,
60 "Shape paramter", alpha);
61
62
63 OperandsAndPartials<T_y, T_loc, T_scale, T_shape>
64 operands_and_partials(y, mu, sigma, alpha);
65
66 using stan::math::SQRT_2;
67 using stan::math::pi;
68 using std::log;
69 using std::exp;
70
71 VectorView<const T_y> y_vec(y);
72 VectorView<const T_loc> mu_vec(mu);
73 VectorView<const T_scale> sigma_vec(sigma);
74 VectorView<const T_shape> alpha_vec(alpha);
75 size_t N = max_size(y, mu, sigma, alpha);
76 const double SQRT_TWO_OVER_PI = std::sqrt(2.0 / stan::math::pi());
77
78 for (size_t n = 0; n < N; n++) {
79 const T_partials_return y_dbl = value_of(y_vec[n]);
80 const T_partials_return mu_dbl = value_of(mu_vec[n]);
81 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
82 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
83 const T_partials_return alpha_dbl_sq = alpha_dbl * alpha_dbl;
84 const T_partials_return diff = (y_dbl - mu_dbl) / sigma_dbl;
85 const T_partials_return diff_sq = diff * diff;
86 const T_partials_return scaled_diff = diff / SQRT_2;
87 const T_partials_return scaled_diff_sq = diff_sq * 0.5;
88 const T_partials_return cdf_log_ = 0.5 * erfc(-scaled_diff) - 2
89 * owens_t(diff, alpha_dbl);
90
91 // cdf_log
92 cdf_log += log(cdf_log_);
93
94 // gradients
95 const T_partials_return deriv_erfc = SQRT_TWO_OVER_PI * 0.5
96 * exp(-scaled_diff_sq) / sigma_dbl;
97 const T_partials_return deriv_owens = erf(alpha_dbl * scaled_diff)
98 * exp(-scaled_diff_sq) / SQRT_TWO_OVER_PI / (-2.0 * pi()) / sigma_dbl;
99 const T_partials_return rep_deriv = (-2.0 * deriv_owens + deriv_erfc)
100 / cdf_log_;
101
102 if (!is_constant_struct<T_y>::value)
103 operands_and_partials.d_x1[n] += rep_deriv;
104 if (!is_constant_struct<T_loc>::value)
105 operands_and_partials.d_x2[n] -= rep_deriv;
106 if (!is_constant_struct<T_scale>::value)
107 operands_and_partials.d_x3[n] -= rep_deriv * diff;
108 if (!is_constant_struct<T_shape>::value)
109 operands_and_partials.d_x4[n] += -2.0 * exp(-0.5 * diff_sq
110 * (1.0 + alpha_dbl_sq))
111 / ((1 + alpha_dbl_sq) * 2.0 * pi()) / cdf_log_;
112 }
113
114 return operands_and_partials.value(cdf_log);
115 }
116 }
117 }
118 #endif
119
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::erf
fvar< T > erf(const fvar< T > &x)
Definition: erf.hpp:14
is_constant_struct.hpp
stan::math::owens_t
fvar< T > owens_t(const fvar< T > &x1, const fvar< T > &x2)
Definition: owens_t.hpp:14
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::SQRT_2
const double SQRT_2
The value of the square root of 2, .
Definition: constants.hpp:21
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::skew_normal_cdf_log
return_type< T_y, T_loc, T_scale, T_shape >::type skew_normal_cdf_log(const T_y &y, const T_loc &mu, const T_scale &sigma, const T_shape &alpha)
Definition: skew_normal_cdf_log.hpp:27
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
owens_t.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::erfc
fvar< T > erfc(const fvar< T > &x)
Definition: erfc.hpp:14
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.